"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BaseController_1 = require("./BaseController");
const YousubsCommand_1 = require("../Util/YousubsCommand");
const Auth_1 = require("../Authentication/Auth");
const Gmail_1 = require("../Client/Gmail");
class AuthController extends BaseController_1.BaseController {
    static register(router) {
        const self = new AuthController();
        router.get("/login", self.login.bind(self));
        router.post("/login", self.doLogin.bind(self));
        router.get("/register", self.register.bind(self));
        router.post("/register", self.doRegister.bind(self));
    }
    constructor() {
        super();
    }
    login(req, res, next) {
        this.title = "Login";
        YousubsCommand_1.default.execute("list-auth")
            .then((output) => {
            if (output === "") {
                res.redirect("/register");
            }
            else {
                this.render(req, res, "pages/login", { errors: {} });
            }
        }).catch(console.error);
    }
    doLogin(req, res, next) {
        const { id, password, mailClient } = req.body;
        const errors = {};
        if (!id) {
            errors.id = ["Id is required."];
        }
        if (!password) {
            errors.password = ["Password is required."];
        }
        if (Object.keys(errors).length) {
            return this.render(req, res, "pages/login", {
                input: { id },
                errors
            });
        }
        YousubsCommand_1.default.execute("check-auth", id, password, "-m", mailClient)
            .then((output) => {
            if (output !== "ok") {
                Auth_1.default.login(id, password, mailClient, req.session);
                res.redirect("/");
            }
            else {
                this.render(req, res, "pages/login", {
                    input: { id },
                    errors: {
                        misc: `Invalid login credentials.`
                    }
                });
            }
        })
            .catch((output) => {
            if (output.indexOf("USER_NOT_FOUND") !== -1) {
                this.render(req, res, "pages/login", {
                    input: { id, mailClient },
                    errors: {
                        misc: `User with id '${id}' is not registered on ${mailClient} client.`
                    }
                });
                return;
            }
            if (output.indexOf("AUTH_FAIL") !== -1) {
                this.render(req, res, "pages/login", {
                    input: { id },
                    errors: {
                        misc: `Invalid login credentials.`
                    }
                });
                return;
            }
            throw output;
        })
            .catch(console.error);
    }
    register(req, res, next) {
        this.title = "Register";
        this.render(req, res, "pages/register", {
            errors: {},
            input: {
                mailClient: "gmail"
            }
        });
    }
    doRegister(req, res, next) {
        const data = req.body;
        const errors = {};
        if (!data.id) {
            errors.id = ["ID is required."];
        }
        if (!data.password && !req.session.reg_password) {
            errors.password = ["Password is required."];
        }
        if (data.password && data.password !== data.password_re) {
            errors.password_re = ["Password mismatch."];
        }
        if (!data.mailClient) {
            errors.mailClient = ["Mail client is required."];
        }
        if (!data.password && req.session.reg_password) {
            data.password = req.session.reg_password;
        }
        else if (data.password) {
            req.session.reg_password = data.password;
            req.session.save(() => console.log("saved"));
        }
        if (Object.keys(errors).length) {
            return this.render(req, res, "pages/register", {
                input: data,
                errors
            });
        }
        switch (data.mailClient) {
            case "gmail":
                let credentials;
                if (!data.credentials) {
                    errors.credentials = ["Credentials is required."];
                }
                else {
                    try {
                        credentials = JSON.parse(data.credentials);
                    }
                    catch (e) {
                        errors.credentials = ["Credentials must be a valid JSON."];
                    }
                }
                if (Object.keys(errors).length) {
                    return this.render(req, res, "pages/register", {
                        input: data,
                        errors
                    });
                }
                if (data.token) {
                    credentials.token = data.token;
                    YousubsCommand_1.default.execute("add-auth", data.id, data.password, credentials, "-m", data.mailClient)
                        .then((response) => {
                        Auth_1.default.login(data.id, data.password, data.mailClient, req.session);
                        res.redirect("/");
                    })
                        .catch(console.error);
                }
                else {
                    Gmail_1.default.getTokenLink(credentials)
                        .then((link) => {
                        this.render(req, res, "pages/register", {
                            input: data,
                            token_link: link,
                            errors
                        });
                    })
                        .catch(console.error);
                }
                return;
            default:
                errors.gmailClient = ["Invalid client."];
        }
        if (Object.keys(errors).length) {
            return this.render(req, res, "pages/register", {
                input: data,
                errors
            });
        }
    }
}
exports.AuthController = AuthController;
//# sourceMappingURL=AuthController.js.map