"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const YousubsCommand_1 = require("../Util/YousubsCommand");
const ListenHistoryRepository_1 = require("../Repository/ListenHistoryRepository");
const iohook = require("iohook");
const desktopIdle = require("desktop-idle");
const MAX_IDLE_TIME = 1000 * 60;
class YousubsSocket {
    constructor(socket) {
        this.musicList = [];
        this.atTrack = -1;
        this.socket = socket;
        this.session = socket.request.session;
        socket.on("disconnect", () => YousubsSocket.remove(this));
        setInterval(() => this.checkSession(), this.session.cookie.maxAge);
        if (this.session.user) {
            this.acquireNextList();
            socket.on("set next", this.setNext.bind(this));
            socket.on("save history", this.saveHistory.bind(this));
            socket.on("save like", this.saveLike.bind(this));
            socket.on("remove like", this.removeLike.bind(this));
            this.listenRepo = new ListenHistoryRepository_1.default(this.session.user.id);
            this.session.user.history = this.listenRepo.getHistory();
            this.session.user.likes = this.listenRepo.getLikes();
            socket.emit("history", this.session.user.history);
            socket.emit("likes", this.session.user.likes);
            YousubsSocket.openSockets.push(this);
        }
        else {
            console.error("Socket io initialized without session.");
        }
    }
    static create(socket) {
        const socketInstance = new this(socket);
    }
    static remove(socket) {
        this.openSockets.splice(this.openSockets.indexOf(socket), 1);
    }
    static setNextTrackAll() {
        this.openSockets.forEach((socket) => {
            socket.setNext();
        });
    }
    static likeAll() {
        this.openSockets.forEach((socket) => {
            socket.like();
        });
    }
    static forwardAll() {
        this.openSockets.forEach((socket) => {
            socket.forward();
        });
    }
    static pauseAll() {
        this.openSockets.forEach((socket) => {
            socket.pause();
        });
    }
    checkSession() {
        this.session.regenerate(() => {
            this.socket.emit('loggedout', { timeout: 'true' });
        });
    }
    getSocket() {
        return this.socket;
    }
    setNext() {
        if (!this.session.user) {
            this.socket.emit("loggedout");
            return;
        }
        const lastTrack = this.musicList[this.atTrack];
        Promise.resolve().then(() => {
            ++this.atTrack;
            if (this.atTrack >= this.musicList.length) {
                this.acquireNextList();
            }
            else {
                this.socket.emit("set next", this.musicList[this.atTrack]);
            }
            if (lastTrack) {
                return YousubsCommand_1.default.execute("remove-email", this.session.user.id, this.session.user.password, lastTrack.id, "-m", this.session.user.client);
            }
        })
            .catch(console.error);
    }
    saveLike(like) {
        if (!this.session.user) {
            this.socket.emit("loggedout");
            return;
        }
        if (!this.session.user.likes.find((l) => l.videoId === like.videoId)) {
            this.session.user.likes.push(like);
            this.listenRepo.saveLikes(this.session.user.likes);
        }
    }
    removeLike(like) {
        if (!this.session.user) {
            this.socket.emit("loggedout");
            return;
        }
        const index = this.session.user.likes.findIndex((l) => l.videoId === like.videoId);
        if (index) {
            this.session.user.likes.splice(index, 1);
            this.listenRepo.saveLikes(this.session.user.likes);
        }
    }
    saveHistory(history) {
        if (!this.session.user) {
            this.socket.emit("loggedout");
            return;
        }
        if (!this.session.user.history.find((l) => l.videoId === history.videoId)) {
            this.session.user.history.push(history);
            this.listenRepo.saveHistory(this.session.user.history);
        }
    }
    like() {
        this.socket.emit("like");
    }
    forward() {
        this.socket.emit("forward");
    }
    pause() {
        this.socket.emit("pause");
    }
    acquireNextList() {
        YousubsCommand_1.default.execute("list-emails", this.session.user.id, this.session.user.password, "-m", this.session.user.client)
            .then((output) => {
            try {
                this.musicList = JSON.parse(output);
            }
            catch (e) {
                console.log("Failed parsing json: ");
                console.log(output);
                throw e;
            }
            this.atTrack = -1;
            this.setNext();
        })
            .catch(console.error);
    }
}
YousubsSocket.openSockets = [];
exports.default = YousubsSocket;
iohook.registerShortcut([29, 56, 41], () => {
    YousubsSocket.setNextTrackAll();
});
iohook.registerShortcut([29, 56, 61001], () => {
    YousubsSocket.forwardAll();
});
iohook.registerShortcut([29, 56, 55], () => {
    YousubsSocket.likeAll();
});
iohook.start();
setInterval(() => {
    if (desktopIdle.getIdleTime() * 1000 > MAX_IDLE_TIME) {
        YousubsSocket.pauseAll();
    }
}, 1000 * 5);
//# sourceMappingURL=YousubsSocket.js.map